#!/bin/bash
# Install script updated 2/5/23

#Ensure there is a Version.txt
if [ -f Version.txt ]; then
    echo -e "Beginning install"
else
    echo -e "Missing Version.txt in the install package folder"
    echo -e "Please contact support@mo-sys.com for help with this"
    exit 1
fi

#Platform check - check not mixing atom with non-atom
sw_for_atom=`grep -m 1 'Compilation Linux kernel =' ./Version.txt | grep -c 'upboard'`
if [[ $(uname -a | grep -c 'upboard') == $sw_for_atom ]]; then
  # all good
  echo -e ''
else
  echo -e 'This software is for a different platform'
  echo -e "Please contact support@mo-sys.com for help with this"
  exit 1
fi

#Platform check - check not mixing arm with non-arm
sw_for_arm=`grep -m 1 'Compilation Linux kernel =' ./Version.txt | grep -c 'v7l'`
if [[ $(uname -a | grep -c 'v7l') == $sw_for_arm ]]; then
  # all good
  echo -e ''
else
  echo -e 'This software is for a different platform'
  echo -e "Please contact support@mo-sys.com for help with this"
  exit 1
fi


#Check OS version for this StarTracker variant
if [[ $(lsb_release -d | grep -c '20.04') == 1 ]]; then

   version=2004
   runtime="/home/martin/runtime"

   if [[ $(grep -c 'Compiled for pre-18.04 Ubuntu' ./Version.txt) == 1 ]]; then

      echo -e 'Version incompatibility: this software is for an older platform'
      echo -e "Please contact support@mo-sys.com for help with this"
      exit 1
   fi

elif [[ $(lsb_release -d | grep -c '14.04') == 1 ]]; then

   version=1404
   runtime="/home/martin/STracker/runtime"

   if [[ $(grep -c 'Compiled for Ubuntu 18.04-20.04' ./Version.txt) == 1 ]]; then

      echo -e 'Version incompatibility: this software is for a newer platform'
      echo -e "Please contact support@mo-sys.com for help with this"
      exit 1
   fi

elif [[ $(lsb_release -d | grep -c 'Raspbian') == 1 ]]; then

   version=10    #Buster
   runtime="/runtime"

else

   echo -e "Unknown OS version (expected 20.04 or 14.04):"
   lsb_release -d
   echo -e "Please contact support@mo-sys.com for help with this"
   exit 1

fi
echo -e "version: $version"


#Ensure the destination has a bin folder
if [[ -d "$runtime/bin" ]]; then
    echo Found bin folder
else
    echo "Could not find bin folder - aborting install"
    echo -e "$runtime/bin"
    exit 1
fi


#Ensure the user is aware they will need a license file for the new s/w to work
#Ignore this check if the user has a license file with what appears to be a license key
#For now exploit the fact that license keys start with 0
if [[ -f $runtime/bin/license.cfg ]]
then
    echo ""
else
	echo -e "The new software needs a license key to work (Error 36 if license invalid)"
	echo -e "You should only install this software if you have received a license,"
	echo -e "are prepared to wait for license help from Mo-Sys support"
	echo -e "or a Mo-Sys technician is standing by to help you install a license"
	echo -e ""
	echo -e "Do you still wish to proceed with the install? (y/n)"
	read
	if [[ "$REPLY" == "y" ]]; then
        echo ""
	else
		echo -e "Install aborted"
		exit 1;
	fi
fi


#Check whether Pylon is needed (or StarTracker won't start)
needs_pylon=`grep -c 'Pylon supported' ./Version.txt`
if [[ $needs_pylon > 0 ]]; then
    if [[ $(locate -c libpylonc.so) == 0 ]]; then
        echo -e "This software requires a missing library (Pylon) - aborting install"
        echo -e "Contact support@mo-sys.com for help"
        exit 1
    else
        echo -e "Pylon needed and found on this system"
    fi
else
    echo -e "No Pylon libraries needed"
fi


#Check whether Peak is needed (or StarTracker won't start)
needs_peak=`grep -c 'PEAK supported' ./Version.txt`
if [[ $needs_peak > 0 ]]; then
    if [[ $(locate -c libids_peak.so) == 0 ]]; then
        echo -e "This software requires a missing library (Peak) - aborting install"
        echo -e "Contact support@mo-sys.com for help"
        exit 1
    else
        echo -e "Peak needed and found on this system"
    fi
else
    echo -e "No Peak libraries needed"
fi


#Check whether avahi-publish is needed for older StarTrackers
if [[ $version -eq 2004 ]]; then
  if [[ $(locate -c avahi-publish) == 0 ]]; then
    if [[ $(grep -c " / ext4 rw" /proc/mounts) == 1 ]]; then
      echo -e "Need avahi-publish: attempting to install..."
      echo 1234.com | sudo -S dpkg -i avahi-utils_0.7-4ubuntu7_amd64.deb
    else
      echo -e "Need avahi-publish: please re-install in Engineering mode"
    fi
  fi #locate avahi-publish
fi #2004 check


#Force an update of Pangolin to the correct version
if [[ $version -eq 2004 ]]; then
 
  #Check size of pangolin to ensure it's the correct version
  pangosize=$(wc -c "/usr/local/lib/libpangolin.so" | awk '{print $1}')
  printf "pango size: %d\n" $pangosize
 
  if [[ $pangosize -ne 2714392 ]]; then
 
   if [[ $(grep -c " / ext4 rw" /proc/mounts) == 1 ]]; then

      echo -e "pangolin not found! Install? (y/n) \c"
      read
      
      if [[ "$REPLY" == "y" ]]; then
       
	     if [ -f ./libpangolin.so ] && [ -s ./libpangolin.so ]
         then 

	        #Update of Pangolin
	        echo 1234.com | sudo -S cp -v -b ./libpangolin.so /usr/local/lib/libpangolin.so
	        echo 1234.com | sudo -S ldconfig
            echo -e "Updating cache - please wait, this could take a few minutes..."
	        echo 1234.com | sudo -S updatedb

         else
            echo -e "Could not find libpangolin.so here - aborting install"
            exit 1
         fi

       else

	     echo -e "libpangolin.so update is required - aborting software install"
	     echo -e "Contact support@mo-sys.com for help"
	     exit 1

       fi

    else

      echo -e "Reboot in engineering mode and try again (pango)"
      exit 1

    fi

  else  #Size is correct
    echo -e "Have latest libpangolin"
  fi

fi  #2004 check



#Check libblob_detector exists or stracker won't start
if [[ $version != 1404 || -f $runtime/lib/libblob_detector.so || -s $runtime/lib/libblob_detector.so ]]; then
 echo "Version not 14.04 or found libblob_detector.so"
else

 echo -e "libblob_detector does not appear to be installed"
 echo -e "Updating cache to confirm - please wait..."
 echo -e "[Check README.txt file for password if asked for one]"
 echo 1234.com | sudo -S updatedb

 #We need to install libblob_detector - check it is here so it can be installed
 if [ -f ./libblob_detector.so ] && [ -s ./libblob_detector.so ]
 then
  echo "Found libblob_detector here as required to install"
 else
  echo "No libblob_detector here (required) - aborting install"
  echo "Contact support@mo-sys.com for this file"
  exit 1
 fi

 if [[ $(grep -c " / ext4 rw" /proc/mounts) == 1 ]]; then

   echo -e "libblob_detector not found! Install? (y/n) \c"
   read
   if [[ "$REPLY" == "y" ]]; then
      
	if [ -f ./libblob_detector.so ] && [ -s ./libblob_detector.so ]
        then

	  #First-time installation of libblob_detector.so
	  echo 1234.com | sudo -S cp -v -b ./libblob_detector.so /usr/local/lib/libblob_detector.so
	  echo 1234.com | sudo -S ldconfig
          echo -e "Updating cache - please wait, this could take a few minutes..."
	  echo 1234.com | sudo -S updatedb

        else
          echo -e "Could not find libblob_detector.so here - aborting install"
          exit 1
        fi

   else

	echo -e "libblob_detector.so is required - aborting software install"
	echo -e "Contact support@mo-sys.com for help"
	exit 1

   fi

 else

   #Need to be in engineering mode to update ldconfig database
   echo -e "Reboot in engineering mode and try again (blob detector)"
   exit 1

 fi

fi


#Check jsoncpp was successfully installed
if [[  $version == 1404  &&  $(locate libjsoncpp.so | head -c1 | wc -c) == 0  ]]
then
   #For some reason no libjsoncpp
   echo -e "jsoncpp needed but could not be installed - aborting"
   exit 1
fi

#Check libblob_detector was successfully installed
if [[  $version == 1404  &&  $(locate libblob_detector.so | head -c1 | wc -c) == 0  ]]
then
   #For some reason the above could not install the missing libblob_detector
   echo -e "libblob_detector needed but could not be installed - aborting"
   exit 1
fi


#Handle T265 reader install
#if no librealsense2 it simply won't run
if [[ $version == 2004 ]] && [ -f ./t265_track_x64 ] && [ -s ./t265_track_x64 ]; then
    echo -e "installing T265 reader"
    install_T265_reader=1
else
    echo -e "not installing T265 reader"
    install_T265_reader=0
fi

#Handle DS80 reader install
#if no xvsdk it simply won't run
if [[ $version == 2004 ]] && [ -f ./DS80_track_x64 ] && [ -s ./DS80_track_x64 ]; then
    echo -e "installing DS80 reader"
    install_DS80_reader=1
else
    echo -e "not installing DS80 reader"
    install_DS80_reader=0
fi

#Handle browser-based UI update
#Is there an assets folder to replace during the install?
#This ensures all irrelevant assets files are deleted.
if [[ -d "$runtime/webserver/assets" ]] && [[ -d "webserver/assets" ]]; then
    echo -e "Deleting webserver assets folder before installing new one"
    delete_assets_fldr=1
else
    echo -e "No webserver assets folder present (locally or in install folder)"
    delete_assets_fldr=0
fi

#Do the install
if [[ 1 ]]; then

   echo -e "jsoncpp & libblob_detector found - proceeding"
   
   #Do the install here!
   if [ -f ./stracker ] && [ -s ./stracker ]
   then
     echo Found stracker
   else
     echo No stracker
     exit 1
   fi

   echo "Copying files to destination bin and new folders and making executable..."

   cp -v -b ./stracker  $runtime/bin/stracker
   chmod +x $runtime/bin/stracker
   ln -sf $runtime/bin/stracker $runtime/bin/Stracker

   if [[ $install_T265_reader == 1 ]]; then
     cp -v -b ./t265_track_x64  $runtime/bin/t265_track
     chmod +x $runtime/bin/t265_track
   fi
   if [[ $install_DS80_reader == 1 ]]; then
     cp -v -b ./DS80_track_x64  $runtime/bin/DS80_track
     chmod +x $runtime/bin/DS80_track
   fi

   if [ -d $runtime/new ]
   then
     echo Found new folder
   else
     echo Making new folder
     mkdir $runtime/new
   fi

   cp -v -b ./stracker       $runtime/new/stracker
   chmod +x $runtime/new/stracker
   
   if [[ $install_T265_reader == 1 ]]; then
     cp -v -b ./t265_track_x64  $runtime/new/t265_track
     chmod +x $runtime/new/t265_track
   fi
   if [[ $install_DS80_reader == 1 ]]; then
     cp -v -b ./DS80_track_x64  $runtime/new/DS80_track
     chmod +x $runtime/new/DS80_track
   fi

   #Copy default backup settings for this release
   cp -rv ./backup $runtime

   #Copy latest images
   cp -rv ./images $runtime
   
   #Copy latest web UI files
   # Delete existing folders first?
   if [[ $delete_assets_fldr == 1 ]]; then
        rm -rv "$runtime/webserver/assets" 
   fi
   cp -rv ./webserver $runtime

   echo "Installation SUCCESSFUL - finished"

fi
